package cn.com.qmth.examcloud.core.examwork.api.controller;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.helpers.poi.ExcelReader;
import cn.com.qmth.examcloud.commons.helpers.poi.ExcelWriter;
import cn.com.qmth.examcloud.commons.util.PathUtil;
import cn.com.qmth.examcloud.core.examwork.service.ExamCourseService;
import cn.com.qmth.examcloud.core.examwork.service.bean.ExamCourseRelationInfo;
import cn.com.qmth.examcloud.core.examwork.service.bean.ExamCourseRelationQuery;
import cn.com.qmth.examcloud.web.config.SystemProperties;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import cn.com.qmth.examcloud.web.support.Naked;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

@RestController
@Api(tags = "考试课程相关接口")
@RequestMapping("${$rmp.ctr.examwork}/exam/course")
public class ExamCourseController extends ControllerSupport {

    @Autowired
    private ExamCourseService examCourseService;

    @Autowired
    private SystemProperties systemConfig;

    @PostMapping("/list")
    @ApiOperation(value = "查询考试相关课程列表(分页)")
    public Page<ExamCourseRelationInfo> list(@RequestBody ExamCourseRelationQuery req) {
        return examCourseService.getExamCourseRelationListByPage(req);
    }

    @PostMapping("/settingScoreLine")
    @ApiOperation(value = "考试课程的及格线设置")
    public void settingScoreLine(@RequestParam Long examId, @RequestParam Long courseId
            , @RequestParam Integer passScoreLine, @RequestParam Integer goodScoreLine) {
        examCourseService.settingScoreLine(examId, courseId, passScoreLine, goodScoreLine);
    }

    @PostMapping("/weixinAnswerEnabled")
    @ApiOperation(value = "（批量）启用或禁用微信小程序作答")
    public void weixinAnswerEnabled(@RequestParam Long examId, @RequestParam Long[] courseIds, @RequestParam Boolean enabled) {
        examCourseService.batchUpdateWeixinAnswerEnabled(examId, courseIds, enabled);
    }

    @ApiOperation(value = "导入-课程的微信小程序作答设置")
    @PostMapping("/import/weixinAnswerEnabledSetting")
    public void importSetting(@RequestParam Long examId, @RequestPart(value = "file", required = false) MultipartFile file) {
        if (file == null) {
            throw new StatusException("文件不能为空！");
        }

        List<String[]> lines;
        try {
            lines = ExcelReader.readSheetBySax(file.getInputStream(), 1, 3);
        } catch (Exception e) {
            throw new StatusException("Excel解析失败！");
        }

        if (CollectionUtils.isEmpty(lines)) {
            throw new StatusException("Excel无内容！");
        }

        for (int n = 1; n < lines.size(); n++) {
            // 默认跳过第一行
            String[] values = lines.get(n);
            String courseCode = values[0];
            boolean enabled = "是".equals(values[2]);

            if (StringUtils.isBlank(courseCode)) {
                continue;
            }

            examCourseService.updateWeixinAnswerEnabled(examId, courseCode, enabled);
        }
    }

    @Naked
    @ApiOperation(value = "导出-课程的微信小程序作答设置")
    @GetMapping("/export/weixinAnswerEnabledSetting")
    public void exportSetting(@RequestParam Long examId, @RequestParam(required = false) Boolean weixinAnswerEnabled) {
        List<ExamCourseRelationInfo> list = examCourseService.getExamCourseRelationList(examId, weixinAnswerEnabled);

        List<Object[]> lines = new ArrayList<>();
        for (ExamCourseRelationInfo info : list) {
            lines.add(new Object[]{info.getCourseCode(), info.getCourseName(), info.getWeixinAnswerEnabled() ? "是" : "否"});
        }

        String filePath = systemConfig.getTempDataDir() + File.separator + System.currentTimeMillis() + ".xlsx";
        File file = new File(filePath);

        ExcelWriter.write(new String[]{"课程代码", "课程名称", "是否启用微信小程序作答"},
                new Class[]{String.class, String.class, String.class},
                lines, new File(filePath));

        exportFile("result.xlsx", file);
        FileUtils.deleteQuietly(file);
    }

    @Naked
    @ApiOperation(value = "下载-课程的微信小程序作答设置模板")
    @GetMapping("/download/weixinAnswerEnabledTemplate")
    public void downloadTemplate() {
        String resourcePath = PathUtil.getResoucePath("templates/weixinAnswerEnabledTemplate.xlsx");
        exportFile("课程的微信小程序作答设置模板.xlsx", new File(resourcePath));
    }

}
