package cn.com.qmth.examcloud.core.examwork.api.controller;

import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.commons.helpers.poi.ExcelWriter;
import cn.com.qmth.examcloud.core.examwork.service.ExamSkipFaceService;
import cn.com.qmth.examcloud.core.examwork.service.bean.ExamSkipFaceInfo;
import cn.com.qmth.examcloud.core.examwork.service.bean.ExamSkipFaceQuery;
import cn.com.qmth.examcloud.web.config.SystemProperties;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import cn.com.qmth.examcloud.web.support.Naked;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.io.FileUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.*;

import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@RestController
@Api(tags = "考试跳过人脸名单相关接口")
@RequestMapping("${$rmp.ctr.examwork}/exam")
public class ExamSkipFaceController extends ControllerSupport {

    @Autowired
    private ExamSkipFaceService examSkipFaceService;

    @Autowired
    private SystemProperties systemConfig;

    @PostMapping("/skip/face/list")
    @ApiOperation(value = "查询跳过人脸名单列表(分页)")
    public Page<ExamSkipFaceInfo> list(@RequestBody ExamSkipFaceQuery req) {
        return examSkipFaceService.getExamSkipFaceList(req);
    }

    @Naked
    @ApiOperation(value = "导出-跳过人脸名单")
    @GetMapping("/skip/face/list/export")
    public void export(ExamSkipFaceQuery req) {
        List<ExamSkipFaceInfo> list = examSkipFaceService.getAllExamSkipFaceList(req);

        List<Object[]> lines = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(list)) {
            for (ExamSkipFaceInfo info : list) {
                lines.add(new Object[]{info.getExamStudentId(), info.getStudentName(), info.getStudentCode(),
                        info.getIdentityNumber(), info.getExamName(), info.getCourseCode(), info.getCourseName(),
                        info.getOrgName(), info.getOperateName(), info.getUpdateTime()});
            }
        }

        String filePath = systemConfig.getTempDataDir() + File.separator + System.currentTimeMillis() + ".xlsx";
        File file = new File(filePath);

        ExcelWriter.write(new String[]{"ID", "考生姓名", "学号", "身份证号", "考试名称", "课程代码", "课程名称", "学习中心", "操作者", "操作时间"},
                new Class[]{Long.class, String.class, String.class, String.class, String.class,
                        String.class, String.class, String.class, String.class, Date.class},
                lines, new File(filePath));

        exportFile("result.xlsx", file);
        FileUtils.deleteQuietly(file);
    }

    @PostMapping("/skip/face/setting")
    @ApiOperation(value = "设置跳过人脸名单（批量启用或取消）")
    public void enabled(@RequestParam Long examId,
                        @RequestParam Long[] examStudentIds,
                        @RequestParam Boolean enabled) {
        User user = super.getAccessUser();
        examSkipFaceService.settingExamSkipFace(examId, examStudentIds, enabled, user.getUserId());
    }

    @PostMapping("/skip/face/check")
    @ApiOperation(value = "检查当前考生是否跳过人脸识别")
    public boolean check(@RequestParam Long examId,
                         @RequestParam Long examStudentId) {
        return examSkipFaceService.checkExamSkipFace(examId, examStudentId);
    }

}
