package cn.com.qmth.examcloud.core.examwork.api.controller;

import cn.com.qmth.examcloud.api.commons.enums.ExamStageStartExamStatus;
import cn.com.qmth.examcloud.api.commons.enums.ExamType;
import cn.com.qmth.examcloud.api.commons.enums.SubmitType;
import cn.com.qmth.examcloud.api.commons.exchange.PageInfo;
import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.helpers.poi.ExcelWriter;
import cn.com.qmth.examcloud.commons.util.DateUtil;
import cn.com.qmth.examcloud.commons.util.RegExpUtil;
import cn.com.qmth.examcloud.core.examwork.api.controller.bean.ExamDomain;
import cn.com.qmth.examcloud.core.examwork.api.controller.bean.ExamStageDomain;
import cn.com.qmth.examcloud.core.examwork.api.controller.bean.ExamStageSettingDomain;
import cn.com.qmth.examcloud.core.examwork.dao.ExamStageRepo;
import cn.com.qmth.examcloud.core.examwork.dao.ExamStageSettingRepo;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamEntity;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamStageEntity;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamStageSettingEntity;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamStudentEntity;
import cn.com.qmth.examcloud.core.examwork.service.ExamStageService;
import cn.com.qmth.examcloud.core.examwork.service.ExamStageSettingService;
import cn.com.qmth.examcloud.core.examwork.service.ExamStudentService;
import cn.com.qmth.examcloud.core.examwork.service.cache.ExamStageCache;
import cn.com.qmth.examcloud.support.cache.CacheHelper;
import cn.com.qmth.examcloud.support.cache.bean.ExamSettingsCacheBean;
import cn.com.qmth.examcloud.web.config.SystemProperties;
import cn.com.qmth.examcloud.web.helpers.GlobalHelper;
import cn.com.qmth.examcloud.web.jpa.PageUtils;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import com.google.common.collect.Lists;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Example;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;
import javax.persistence.criteria.Subquery;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @Description 场次
 * @Author lideyin
 * @Date 2020/7/22 15:00
 * @Version 1.0
 */
@RestController
@Api(tags = "场次相关接口")
@RequestMapping("${$rmp.ctr.examwork}/examStage")
public class ExamStageController extends ControllerSupport {

    private static final String[] EXAM_STAGE_EXCEL_HEADER = new String[]{"场次ID", "场次号",
            "开始时间", "结束考试时间", "控制设置", "状态"};

    @Autowired
    ExamStageService examStageService;

    @Autowired
    ExamStageSettingService examStageSettingService;

    @Autowired
    ExamStageRepo examStageRepo;

    @Autowired
    ExamStageSettingRepo examStageSettingRepo;

    @Autowired
    ExamStudentService examStudentService;

    @Autowired
    SystemProperties systemConfig;
    @Autowired
    ExamStageCache examStageCache;

    /**
     * 分页查询场次列表
     *
     * @param curPage    页索引
     * @param pageSize   页大小
     * @param examId     考试id
     * @param stageOrder 场次号
     * @param startTime  允许考试开始时间
     * @param endTime    允许考试结束时间
     * @param enable     是否启用
     * @return
     */
    @ApiOperation(value = "分页查询场次")
    @GetMapping("queryPage/{curPage}/{pageSize}")
    public PageInfo<ExamStageDomain> queryPage(@PathVariable Integer curPage,
                                               @PathVariable Integer pageSize,
                                               @RequestParam Long examId,
                                               @RequestParam(required = false) Integer stageOrder,
                                               @RequestParam(required = false) String startTime,
                                               @RequestParam(required = false) String endTime,
                                               @RequestParam(required = false) Boolean enable) {
        User accessUser = getAccessUser();

        Specification<ExamStageEntity> specification = (root, query, cb) -> {
            List<Predicate> predicates = new ArrayList<>();
            predicates.add(cb.equal(root.get("rootOrgId"), accessUser.getRootOrgId()));
            predicates.add(cb.equal(root.get("examId"), examId));
            if (null != stageOrder) {
                predicates.add(cb.equal(root.get("stageOrder"), stageOrder));
            }
            if (StringUtils.isNotBlank(startTime)) {
                predicates.add(cb.greaterThanOrEqualTo(root.get("startTime"),
                        DateUtil.parse(startTime, DateUtil.DatePatterns.CHINA_DEFAULT)));
            }
            if (StringUtils.isNotBlank(endTime)) {
                predicates.add(cb.lessThanOrEqualTo(root.get("endTime"),
                        DateUtil.parse(endTime, DateUtil.DatePatterns.CHINA_DEFAULT)));
            }
            if (null != enable) {
                predicates.add(cb.equal(root.get("enable"), enable));
            }

            return cb.and(predicates.toArray(new Predicate[predicates.size()]));
        };

        PageRequest pageRequest = PageRequest.of(curPage, pageSize,
                Sort.by(Sort.Direction.DESC, "stageOrder"));

        Page<ExamStageEntity> page = examStageRepo.findAll(specification, pageRequest);

        Iterator<ExamStageEntity> iterator = page.iterator();
        List<ExamStageDomain> list = Lists.newArrayList();

        while (iterator.hasNext()) {
            ExamStageEntity next = iterator.next();
            ExamStageDomain domain = new ExamStageDomain();
            domain.setId(next.getId());
            domain.setRootOrgId(next.getRootOrgId());
            domain.setExamId(next.getExamId());
            domain.setStageOrder(next.getStageOrder());
            domain.setStartTime(next.getStartTime());
            domain.setEndTime(next.getEndTime());
            domain.setEnable(next.getEnable());
            domain.setSpecialSetting(next.getSpecialSetting());
            domain.setSubmitType(next.getSubmitType());
            domain.setSubmitDuration(next.getSubmitDuration() == null ? 0 : next.getSubmitDuration());
            domain.setUpdateTime(next.getUpdateTime());
            domain.setCreationTime(next.getCreationTime());

            list.add(domain);

        }

        PageInfo<ExamStageDomain> ret = PageUtils.toPageInfo(page, list);
        return ret;
    }

    /**
     * 查询场次
     *
     * @param name
     * @param enable
     * @param examId
     * @param rootOrgId
     * @return
     * @author WANGWEI
     */
    @ApiOperation(value = "查询考试批次")
    @GetMapping("queryByNameLike")
    public List<ExamStageDomain> query(@RequestParam(required = true) String name,
                                       @RequestParam(required = true) Long examId,
                                       @RequestParam(required = false) Boolean enable,
                                       @RequestParam(required = false) Long rootOrgId) {

        if (null == rootOrgId) {
            rootOrgId = getRootOrgId();
        }
        final Long finalRootOrgId = rootOrgId;

        //如果场次号不为数字，则直接返回空集合
        if (StringUtils.isNotEmpty(name) && !StringUtils.isNumeric(name)) {
            return Lists.newArrayList();
        }

        Specification<ExamStageEntity> specification = (root, query, cb) -> {
            List<Predicate> predicates = new ArrayList<>();
            predicates.add(cb.equal(root.get("rootOrgId"), finalRootOrgId));
            predicates.add(cb.equal(root.get("examId"), examId));
            if (StringUtils.isNotBlank(name)) {
                predicates.add(cb.equal(root.get("stageOrder"), Long.valueOf(name)));
            }
            if (null != enable) {
                predicates.add(cb.equal(root.get("enable"), enable));
            }

            return cb.and(predicates.toArray(new Predicate[predicates.size()]));
        };

        PageRequest pageRequest = PageRequest.of(0, 100,
                Sort.by(Sort.Direction.ASC, "stageOrder"));
        Page<ExamStageEntity> page = examStageRepo.findAll(specification, pageRequest);

        Iterator<ExamStageEntity> iterator = page.iterator();
        List<ExamStageDomain> resultList = Lists.newArrayList();

        while (iterator.hasNext()) {
            ExamStageEntity next = iterator.next();
            ExamStageDomain bean = new ExamStageDomain();
            bean.setId(next.getId());
            bean.setRootOrgId(next.getRootOrgId());
            bean.setExamId(next.getExamId());
            bean.setStageOrder(next.getStageOrder());
            bean.setStartTime(next.getStartTime());
            bean.setEndTime(next.getEndTime());
            bean.setEnable(next.getEnable());
            bean.setSpecialSetting(next.getSpecialSetting());
            bean.setSubmitType(next.getSubmitType());
            bean.setSubmitDuration(next.getSubmitDuration());
            bean.setUpdateTime(next.getUpdateTime());
            bean.setCreationTime(next.getCreationTime());

            resultList.add(bean);
        }

        return resultList;
    }

    @PostMapping(value = "batchAddStage")
    @ApiOperation(value = "添加场次")
    @Transactional
    public void batchAddStage(@RequestBody List<ExamStageDomain> examStages) {
        if (examStages.size() == 0) {
            throw new StatusException("10001", "examStages参数不允许为空");
        }

        Long examId = examStages.get(0).getExamId();
        if (null == examId) {
            throw new StatusException("100002", "考试id不允许为空");
        }

        User accessUser = getAccessUser();
        ExamSettingsCacheBean examSettings = CacheHelper.getExamSettings(examId);
        if (!examSettings.getHasValue()) {
            throw new StatusException("100003", "考试id不正确");
        }
        Date examStartTime = examSettings.getBeginTime();
        Date examEndTime = examSettings.getEndTime();

        //起始场次号
        int startStageOrder = 0;
        ExamStageEntity maxStageOrderEntity = examStageRepo.findFirstByExamIdOrderByStageOrderDesc(examId);
        if (null != maxStageOrderEntity) {
            startStageOrder = maxStageOrderEntity.getStageOrder();
        }
        for (ExamStageDomain esd : examStages) {
            startStageOrder++;

            if (null == esd.getStartTime()) {
                throw new StatusException("100005", "进入考试开始时间不允许为空");
            }

            if (null == esd.getEndTime()) {
                throw new StatusException("100006", "进入考试结束时间不允许为空");
            }

            if (esd.getStartTime().after(esd.getEndTime())) {
                throw new StatusException("100007", "进入考试开始时间不得晚于结束时间");
            }

            if (esd.getStartTime().before(examStartTime) || esd.getStartTime().after(examEndTime)) {
                throw new StatusException("100008", "进入考试开始时间不得超过考试批次中设置的时间范围");
            }

            if (esd.getEndTime().before(examStartTime) || esd.getEndTime().after(examEndTime)) {
                throw new StatusException("100009", "进入考试开始时间不得超过考试批次中设置的时间范围");
            }

            ExamStageEntity esEntity = new ExamStageEntity();
            esEntity.setRootOrgId(accessUser.getRootOrgId());
            esEntity.setExamId(examId);
            esEntity.setStageOrder(startStageOrder);
            esEntity.setStartTime(esd.getStartTime());
            esEntity.setEndTime(esd.getEndTime());

            if (null == esd.getEnable()) {
                esEntity.setEnable(true);
            } else {
                esEntity.setEnable(esd.getEnable());
            }

            if (null == esd.getSpecialSetting()) {
                esEntity.setSpecialSetting(false);
            } else {
                esEntity.setSpecialSetting(esd.getSpecialSetting());
            }

            if (null == esd.getSubmitType()) {
                esEntity.setSubmitType(SubmitType.NORMAL);
            } else {
                esEntity.setSubmitType(esd.getSubmitType());

                if (SubmitType.TIMING_END == esd.getSubmitType()) {
                    if (null == esd.getSubmitDuration()) {
                        throw new StatusException("100007", "定点交卷时长不允许为空");
                    }
                }
            }

            esEntity.setSubmitDuration(esd.getSubmitDuration());

            esEntity.setStartExamStatus(ExamStageStartExamStatus.NOT_START);
            examStageRepo.save(esEntity);
        }
    }

    @PutMapping(value = "modifyStage")
    @ApiOperation(value = "修改场次信息")
    @Transactional
    public void modifyStage(@RequestParam Long id,
                            @RequestParam(required = false) String startTime,
                            @RequestParam(required = false) String endTime,
                            @RequestParam Boolean specialSetting,
                            @RequestParam(required = false) String submitType,
                            @RequestParam(required = false) Integer submitDuration) {
        if (null == id) {
            throw new StatusException("10001", "场次id不允许为空");
        }

        ExamStageEntity esEntity = GlobalHelper.getEntity(examStageRepo, id, ExamStageEntity.class);
        if (null == esEntity) {
            throw new StatusException("10002", "场次id不正确");
        }

        Long examId = esEntity.getExamId();
        ExamSettingsCacheBean examSettings = CacheHelper.getExamSettings(examId);
        Date examStartTime = examSettings.getBeginTime();
        Date examEndTime = examSettings.getEndTime();

        if ((null != startTime && null == endTime) || (null == startTime && null != endTime)) {
            throw new StatusException("100003", "进入考试开始时间和结束时间必须同时有值");
        }

        if (StringUtils.isNotEmpty(startTime) && StringUtils.isNotEmpty(endTime)) {
            Date st = DateUtil.parse(startTime, DateUtil.DatePatterns.CHINA_DEFAULT);
            Date et = DateUtil.parse(endTime, DateUtil.DatePatterns.CHINA_DEFAULT);
            if (st.after(et)) {
                throw new StatusException("100004", "进入考试开始时间不得晚于结束时间");
            }

            if (st.before(examStartTime) || st.after(examEndTime)) {
                throw new StatusException("100005", "进入考试开始时间不得超过考试批次中设置的时间范围");
            }

            if (et.before(examStartTime) || et.after(examEndTime)) {
                throw new StatusException("100006", "进入考试开始时间不得超过考试批次中设置的时间范围");
            }
            esEntity.setStartTime(st);
            esEntity.setEndTime(et);
        }

        //未进行特殊设置，默认不使用特殊设置
        if (null == specialSetting) {
            specialSetting = false;
        }
        esEntity.setSpecialSetting(specialSetting);

        if (StringUtils.isEmpty(submitType)) {
            submitType = SubmitType.NORMAL.name();
        }
        esEntity.setSubmitType(SubmitType.valueOf(submitType));

        if (specialSetting == true && submitType.equals(SubmitType.TIMING_END.name())) {
            if (null == submitDuration) {
                throw new StatusException("100007", "统一收卷时长不允许为空");
            }

            if (submitDuration.intValue() <= 0) {
                throw new StatusException("100006", "统一收卷时长必须大于0");
            }
            esEntity.setSubmitDuration(submitDuration);
        }

        examStageRepo.save(esEntity);

        //刷新缓存
        examStageCache.refresh(examId, esEntity.getId());
    }

    @PutMapping(value = "modifyStageEnable")
    @ApiOperation(value = "启用/禁用场次")
    @Transactional
    public void modifyStageEnable(@RequestParam List<Long> examStageIds,
                                  @RequestParam Boolean enable) {
        if (examStageIds.size() == 0) {
            throw new StatusException("10001", "examStageIds参数不允许为空");
        }
        for (Long esId : examStageIds) {
            ExamStageEntity entity = GlobalHelper.getEntity(examStageRepo, esId, ExamStageEntity.class);
            entity.setEnable(enable);
            examStageRepo.save(entity);

            //刷新缓存
            examStageCache.refresh(entity.getExamId(), entity.getId());
        }
    }

    @PostMapping(value = "saveStageSetting")
    @ApiOperation(value = "保存通用场次设置")
    @Transactional
    public void saveStageSetting(@RequestBody ExamStageSettingDomain domain) {
        ExamStageSettingEntity essEntity;
        //新增
        if (null == domain.getId()) {
            if (null == domain.getExamId()) {
                throw new StatusException("10001", "考试id不允许为空");
            }

            SubmitType submitType = null == domain.getSubmitType() ? SubmitType.NORMAL : domain.getSubmitType();

            if (SubmitType.TIMING_END == submitType) {
                if (null == domain.getSubmitDuration()) {
                    throw new StatusException("100002", "统一收卷时长不允许为空");
                }

                if (domain.getSubmitDuration().intValue() <= 0) {
                    throw new StatusException("100003", "统一收卷时长必须大于0");
                }
            }

            essEntity = new ExamStageSettingEntity();
            essEntity.setExamId(domain.getExamId());
            essEntity.setSubmitType(submitType);
            essEntity.setSubmitDuration(domain.getSubmitDuration());

            examStageSettingRepo.save(essEntity);
        }
        //修改
        else {
            if (null == domain.getExamId()) {
                throw new StatusException("10004", "考试id不允许为空");
            }

            SubmitType submitType = null == domain.getSubmitType() ? SubmitType.NORMAL : domain.getSubmitType();

            if (SubmitType.TIMING_END == submitType) {
                if (null == domain.getSubmitDuration()) {
                    throw new StatusException("100005", "统一收卷时长不允许为空");
                }

                if (domain.getSubmitDuration().intValue() <= 0) {
                    throw new StatusException("100006", "统一收卷时长必须大于0");
                }
            }


            essEntity = examStageSettingRepo.findByExamId(domain.getExamId());
            essEntity.setSubmitType(domain.getSubmitType());
            essEntity.setSubmitDuration(domain.getSubmitDuration());

            examStageSettingRepo.save(essEntity);
        }

        //刷新缓存
        List<ExamStageEntity> existExamStageList = examStageRepo.findByExamId(domain.getExamId());
        if (null != existExamStageList && !existExamStageList.isEmpty()) {
            existExamStageList.forEach(p -> {
                examStageCache.refresh(domain.getExamId(), p.getId());
            });
        }
    }

    @ApiOperation(value = "获取场次通用设置")
    @GetMapping("getExamStageSetting")
    public ExamStageSettingDomain getExamStageSetting(@RequestParam Long examId) {
        ExamStageSettingEntity essEntity = examStageSettingRepo.findByExamId(examId);
        if (null == essEntity) {
            return null;
        }

        ExamStageSettingDomain result = new ExamStageSettingDomain();
        result.setId(essEntity.getId());
        result.setExamId(essEntity.getExamId());
        result.setSubmitType(essEntity.getSubmitType());
        result.setSubmitDuration(essEntity.getSubmitDuration());

        return result;
    }

    @ApiOperation(value = "清空考试下所有场次")
    @DeleteMapping("clearStages")
    @Transactional
    public void clearStages(@RequestParam Long examId) {
        if (null == examId) {
            throw new StatusException("10001", "examId参数不允许为空");
        }

        List<ExamStageEntity> existExamStageList = examStageRepo.findByExamId(examId);
        if (null == existExamStageList || existExamStageList.isEmpty()) {
            return;
        }

        //如果存在已经开考的场次，则不允许清空
        ExamStageEntity query = new ExamStageEntity();
        query.setExamId(examId);
        query.setStartExamStatus(ExamStageStartExamStatus.STARTED);
        Example<ExamStageEntity> example = Example.of(query);
        boolean existStartedExamData = examStageRepo.exists(example);
        if (existStartedExamData) {
            throw new StatusException("10002", "该考试下已存在已经开考的场次，不允许清空");
        }

        //解绑考生与场次的对应关系
        examStudentService.unbindExamStudentExamStage(examId);
        examStageRepo.deleteByExamId(examId);
        //删除相关缓存
        existExamStageList.forEach(p -> examStageCache.remove(p.getExamId(), p.getId()));
    }

    @ApiOperation(value = "导出场次")
    @GetMapping("exportExamStage/{examId}")
    public void exportExamStage(@PathVariable Long examId, HttpServletResponse response) {
        List<ExamStageEntity> examStageList = examStageRepo.findByExamId(examId);

        if (null == examStageList) {
            examStageList = new ArrayList<>();
        }

        List<Object[]> datas = Lists.newArrayList();

        for (ExamStageEntity cur : examStageList) {
            String startTime = null == cur.getStartTime()
                    ? null
                    : DateUtil.format(cur.getStartTime(), DateUtil.DatePatterns.CHINA_DEFAULT);
            String endTime = null == cur.getEndTime()
                    ? null
                    : DateUtil.format(cur.getEndTime(), DateUtil.DatePatterns.CHINA_DEFAULT);
            String specialSetting = cur.getSpecialSetting() ? "自定义" : "通用";
            String enable = cur.getEnable() ? "启用" : "禁用";
            datas.add(new Object[]{String.valueOf(cur.getId()), String.valueOf(cur.getStageOrder()),
                    startTime, endTime, specialSetting, enable});
        }

        String filePath = systemConfig.getTempDataDir() + File.separator + "examStage_" + examId + "_"
                + System.currentTimeMillis() + ".xlsx";
        File file = new File(filePath);

        ExcelWriter.write(
                EXAM_STAGE_EXCEL_HEADER, new Class[]{String.class, String.class,
                        String.class, String.class, String.class, String.class},
                datas, new File(filePath));

        exportFile("场交设置-" + examId + ".xlsx", file);

        FileUtils.deleteQuietly(file);
    }

}
