package cn.com.qmth.examcloud.core.examwork.api.provider;

import cn.com.qmth.examcloud.api.commons.enums.ExamStageStartExamStatus;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.examwork.dao.ExamStageRepo;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamStageEntity;
import cn.com.qmth.examcloud.examwork.api.ExamStageCloudService;
import cn.com.qmth.examcloud.examwork.api.bean.ExamStageBean;
import cn.com.qmth.examcloud.examwork.api.request.GetExamStageReq;
import cn.com.qmth.examcloud.examwork.api.request.ModifyExamStageStartExamStatusReq;
import cn.com.qmth.examcloud.examwork.api.response.GetExamStageResp;
import cn.com.qmth.examcloud.examwork.api.response.ModifyExamStageStartExamStatusResp;
import cn.com.qmth.examcloud.web.helpers.GlobalHelper;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * @Description 状态码范围：013XXX<br>考试场次相关接口
 * @Author lideyin
 * @Date 2020/8/6 15:53
 * @Version 1.0
 */
@RestController
@Api(tags = "RPC-考试场次相关接口")
@RequestMapping("${$rmp.cloud.examwork}" + "examStage")
public class ExamStageCloudServiceProvider extends ControllerSupport implements ExamStageCloudService {

    private static final long serialVersionUID = -7011233758819589714L;

    @Autowired
    private ExamStageRepo examStageRepo;

    /**
     * 更新场次中的考试状态
     *
     * @param req
     */
    @ApiOperation(value = "更新场次中的考试状态", notes = "")
    @PostMapping("modifyExamStageStartExamStatus")
    @Override
    public ModifyExamStageStartExamStatusResp modifyExamStageStartExamStatus(@RequestBody ModifyExamStageStartExamStatusReq req) {
        ExamStageEntity stageEntity = GlobalHelper.getEntity(examStageRepo, req.getExamStageId(), ExamStageEntity.class);

        if (null == stageEntity) {
            throw new StatusException("013001", String.format("场次id:%s不存在", req.getExamStageId()));
        }

        stageEntity.setStartExamStatus(ExamStageStartExamStatus.valueOf(req.getExamStageStartExamStatus()));
        examStageRepo.save(stageEntity);

        return new ModifyExamStageStartExamStatusResp();
    }

    /**
     * 获取场次信息
     */
    @ApiOperation(value = "根据查询条件获取场次信息", notes = "")
    @PostMapping("getExamStage")
    @Override
    public GetExamStageResp getExamStage(@RequestBody GetExamStageReq req) {
        GetExamStageResp resp = new GetExamStageResp();
        ExamStageEntity esEntity;
        if (null != req.getId()) {
            esEntity = GlobalHelper.getEntity(examStageRepo, req.getId(), ExamStageEntity.class);
            if (null == esEntity) {
                throw new StatusException("013101", String.format("找不到id为:%s的场次信息", req.getId()));
            }

            resp.setId(req.getId());
            resp.setExamStageBean(copyFrom(esEntity));
            return resp;
        }

        if (null == req.getExamId()) {
            throw new StatusException("013102", "考试id不允许为空");
        }

        if (null == req.getExamStageOrder()) {
            throw new StatusException("013103", "场次号不允许为空");
        }

        esEntity = examStageRepo.findByExamIdAndStageOrder(req.getExamId(), req.getExamStageOrder());

        if (null == esEntity) {
            throw new StatusException("013101",
                    String.format("找不到考试id:%s,场次号:%s的场次信息", req.getExamId(), req.getExamStageOrder()));
        }

        resp.setId(esEntity.getId());
        resp.setExamStageBean(copyFrom(esEntity));
        return resp;
    }

    /**
     * 根据实体转化为bean
     *
     * @param esEntity
     * @return
     */
    private ExamStageBean copyFrom(ExamStageEntity esEntity) {
        ExamStageBean bean = new ExamStageBean();
        bean.setId(esEntity.getId());
        bean.setExamId(esEntity.getExamId());
        bean.setRootOrgId(esEntity.getRootOrgId());
        bean.setStageOrder(esEntity.getStageOrder());
        bean.setEnable(esEntity.getEnable());
        return bean;
    }

}
