package cn.com.qmth.examcloud.core.examwork.api.provider;

import cn.com.qmth.examcloud.core.examwork.dao.entity.NoticeRulePublishProgressEntity;
import cn.com.qmth.examcloud.core.examwork.service.NoticeService;
import cn.com.qmth.examcloud.examwork.api.NoticeCloudService;
import cn.com.qmth.examcloud.examwork.api.bean.NoticeRulePublishProgressBean;
import cn.com.qmth.examcloud.examwork.api.request.DisposePublishingUserNoticeReq;
import cn.com.qmth.examcloud.examwork.api.request.UpdateNoticeStatusReq;
import cn.com.qmth.examcloud.examwork.api.response.DisposePublishingUserNoticeResp;
import cn.com.qmth.examcloud.examwork.api.response.GetNoticeRulePublishProgressListResp;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.List;

/**
 * 考试云服务
 *
 * @author WANGWEI
 * @date 2018年11月23日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@RestController
@Api(tags = "RPC-通知相关接口")
@RequestMapping("${$rmp.cloud.examwork}" + "notice")
public class NoticeCloudServiceProvider extends ControllerSupport implements NoticeCloudService {

    private static final long serialVersionUID = 4359829563744893206L;

    @Autowired
    private NoticeService noticeService;

    @ApiOperation(value = "获取待处理的通知进度数据")
    @PostMapping("getToBeDisposedNoticeRulePublishProgressList")
    @Override
    public GetNoticeRulePublishProgressListResp getToBeDisposedNoticeRulePublishProgressList() {
        GetNoticeRulePublishProgressListResp resp = new GetNoticeRulePublishProgressListResp();
        List<NoticeRulePublishProgressEntity> progressEntityList = noticeService
                .getToBeDisposedNoticeRulePublishProgressList();
        if (progressEntityList == null || progressEntityList.isEmpty()) {
            return resp;
        }
        List<NoticeRulePublishProgressBean> beanList = new ArrayList<>();
        for (NoticeRulePublishProgressEntity entity : progressEntityList) {
            NoticeRulePublishProgressBean bean = new NoticeRulePublishProgressBean();
            bean.setId(entity.getId());
            bean.setMaxCommonUserId(entity.getMaxCommonUserId());
            bean.setMaxStudentId(entity.getMaxStudentId());
            bean.setNoticeId(entity.getNoticeId());
            bean.setNoticeReceiverRuleType(entity.getNoticeReceiverRuleType());
            bean.setRootOrgId(entity.getRootOrgId());
            beanList.add(bean);
        }
        resp.setList(beanList);
        return resp;
    }

    @ApiOperation(value = "处理待发布和发布中的数据")
    @PostMapping("disposePublishingUserNotice")
    @Transactional
    @Override
    public DisposePublishingUserNoticeResp disposePublishingUserNotice(
            @RequestBody DisposePublishingUserNoticeReq req) {
        // 处理发布中的通知数据，并返回下一个要处理的用户id
        Long nextUserId = noticeService.disposePublishingUserNotice(req.getStartUserId(),
                copytNoticeRulePublishProgressEntityFrom(req.getNoticeRulePublishProgress()));

        DisposePublishingUserNoticeResp resp = new DisposePublishingUserNoticeResp();
        resp.setNextUserId(nextUserId);
        return resp;
    }

    // 实体转换
    private NoticeRulePublishProgressEntity copytNoticeRulePublishProgressEntityFrom(
            NoticeRulePublishProgressBean processBean) {
        NoticeRulePublishProgressEntity processEntity = new NoticeRulePublishProgressEntity();
        processEntity.setId(processBean.getId());
        processEntity.setMaxCommonUserId(processBean.getMaxCommonUserId());
        processEntity.setMaxStudentId(processBean.getMaxStudentId());
        processEntity.setNoticeId(processBean.getNoticeId());
        processEntity.setNoticeReceiverRuleType(processBean.getNoticeReceiverRuleType());
        processEntity.setRootOrgId(processBean.getRootOrgId());
        return processEntity;
    }

    @ApiOperation(value = "清理过期的通知数据")
    @PostMapping("disposeOverdueNotice")
    @Transactional
    @Override
    public void disposeOverdueNotice() {
        noticeService.disposeOverdueNotice();
    }

    @ApiOperation(value = "更新通知状态")
    @PostMapping("updateNoticeStatus")
    @Transactional
    @Override
    public void updateNoticeStatus(@RequestBody UpdateNoticeStatusReq updateNoticeStatusReq) {
        noticeService.updateNoticeStatus(updateNoticeStatusReq.getNoticeId(),
                updateNoticeStatusReq.getNoticeStatus());
    }

}
