package cn.com.qmth.examcloud.core.examwork.dao;

import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamCourseRelationEntity;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamCourseRelationPK;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.data.repository.query.QueryByExampleExecutor;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;

public interface ExamCourseRelationRepo extends JpaRepository<ExamCourseRelationEntity, ExamCourseRelationPK>,
        QueryByExampleExecutor<ExamCourseRelationEntity>, JpaSpecificationExecutor<ExamCourseRelationEntity> {

    void deleteByExamId(Long examId);

    void deleteByExamIdAndCourseId(Long examId, Long courseId);

    ExamCourseRelationEntity findByExamIdAndCourseCode(Long examId, String courseCode);

    ExamCourseRelationEntity findByExamIdAndCourseId(Long examId, Long courseId);

    @Transactional
    @Modifying
    @Query("update ExamCourseRelationEntity s set s.courseName = ?1, s.courseLevel  = ?2,s.courseEnable=?3 where s.courseId=?4")
    void updateCourse(String courseName, String courseLevel, Boolean courseEnable, Long courseId);

    @Transactional
    @Modifying
    @Query("update ExamCourseRelationEntity set weixinAnswerEnabled=:enabled, updateTime=:updateTime where examId=:examId and courseId in (:courseIds)")
    int batchUpdateWeixinAnswerEnabled(@Param("examId") Long examId, @Param("courseIds") Long[] courseIds,
                                       @Param("enabled") Boolean enabled, @Param("updateTime") Date updateTime);

    @Transactional
    @Modifying
    @Query("update ExamCourseRelationEntity set weixinAnswerEnabled=:enabled, updateTime=:updateTime where examId=:examId and courseCode =:courseCode")
    int updateWeixinAnswerEnabled(@Param("examId") Long examId, @Param("courseCode") String courseCode,
                                  @Param("enabled") Boolean enabled, @Param("updateTime") Date updateTime);

    @Transactional
    @Modifying
    @Query("update ExamCourseRelationEntity set passScoreLine =:passScoreLine, goodScoreLine =:goodScoreLine, updateTime=:updateTime where examId=:examId and courseId=:courseId")
    int updateScoreLine(@Param("examId") Long examId, @Param("courseId") Long courseId
            , @Param("passScoreLine") Integer passScoreLine, @Param("goodScoreLine") Integer goodScoreLine
            , @Param("updateTime") Date updateTime);

    @Query("SELECT courseId from ExamCourseRelationEntity where examId=:examId")
    List<Long> findExamCourseIds(@Param("examId") Long examId);

}
