package cn.com.qmth.examcloud.core.examwork.dao.entity;

import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.Table;

import org.springframework.format.annotation.DateTimeFormat;

import cn.com.qmth.examcloud.api.commons.enums.ExamSpecialSettingsType;
import cn.com.qmth.examcloud.api.commons.enums.ExamType;
import cn.com.qmth.examcloud.web.jpa.JpaEntity;

/**
 * 考试--特殊设置<br>
 * orgId,courseId,studentId仅有一个可以不为空<br>
 * 考试维度: orgId==null && courseId==null && studentId==null<br>
 * 机构维度: orgId!=null && courseId==null && studentId==null<br>
 * 学生维度: orgId==null && courseId==null && studentId!=null<br>
 * 课程维度: orgId==null && courseId!=null && studentId ==null<br>
 *
 * @author WANGWEI
 * @date 2018年5月16日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@Entity
@Table(name = "EC_E_EXAM_SPECIAL_SETTINGS", indexes = {
		@Index(name = "IDX_E_E_S_S_001001", columnList = "examId,courseId,orgId,studentId", unique = true),
		@Index(name = "IDX_E_E_S_S_001002", columnList = "examId,courseId", unique = false),
		@Index(name = "IDX_E_E_S_S_001003", columnList = "examId,orgId", unique = false),
		@Index(name = "IDX_E_E_S_S_001004", columnList = "examId,studentId", unique = false),
		@Index(name = "IDX_E_E_S_S_001005", columnList = "rootOrgId,courseId", unique = false),
		@Index(name = "IDX_E_E_S_S_001006", columnList = "rootOrgId,orgId", unique = false),
		@Index(name = "IDX_E_E_S_S_001007", columnList = "rootOrgId,studentId", unique = false)})
public class ExamSpecialSettingsEntity extends JpaEntity {

	private static final long serialVersionUID = -3335725218626631530L;

	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	private Long id;

	/**
	 * 顶级机构ID
	 */
	@Column(nullable = false)
	private Long rootOrgId;

	/**
	 * 考试ID
	 */
	@Column(nullable = false)
	private Long examId;

	/**
	 * 考试ID
	 */
	@Column(nullable = true)
	private Long courseId;

	/**
	 * 机构ID
	 */
	@Column(nullable = true)
	private Long orgId;

	/**
	 * 学生ID
	 */
	@Column(nullable = true)
	private Long studentId;

	/**
	 * 考试类型
	 */
	@Column(nullable = false)
	@Enumerated(EnumType.STRING)
	private ExamType examType;

	/**
	 * 考试是否可用
	 */
	@Column(nullable = false)
	private Boolean examEnable;

	/**
	 * 考试批次开始时间
	 */
	@DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	private Date beginTime;

	/**
	 * 考试批次结束时间
	 */
	@DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	private Date endTime;

	/**
	 * 是否禁止考试
	 */
	@Column(nullable = false)
	private Boolean examLimit;

	/**
	 * 开启特殊设置
	 */
	@Column(nullable = false)
	private Boolean specialSettingsEnabled;

	/**
	 * 特殊设置类型
	 */
	@Column(nullable = true)
	@Enumerated(EnumType.STRING)
	private ExamSpecialSettingsType specialSettingsType;

	/**
	 * 扩展属性1
	 */
	private String ext1;

	/**
	 * 扩展属性2
	 */
	private String ext2;

	/**
	 * 扩展属性3
	 */
	private String ext3;

	/**
	 * 扩展属性4
	 */
	private String ext4;

	/**
	 * 扩展属性5
	 */
	private String ext5;

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public Long getRootOrgId() {
		return rootOrgId;
	}

	public void setRootOrgId(Long rootOrgId) {
		this.rootOrgId = rootOrgId;
	}

	public Long getExamId() {
		return examId;
	}

	public void setExamId(Long examId) {
		this.examId = examId;
	}

	public Long getCourseId() {
		return courseId;
	}

	public void setCourseId(Long courseId) {
		this.courseId = courseId;
	}

	public Long getOrgId() {
		return orgId;
	}

	public void setOrgId(Long orgId) {
		this.orgId = orgId;
	}

	public Long getStudentId() {
		return studentId;
	}

	public void setStudentId(Long studentId) {
		this.studentId = studentId;
	}

	public ExamType getExamType() {
		return examType;
	}

	public void setExamType(ExamType examType) {
		this.examType = examType;
	}

	public Boolean getExamEnable() {
		return examEnable;
	}

	public void setExamEnable(Boolean examEnable) {
		this.examEnable = examEnable;
	}

	public Date getBeginTime() {
		return beginTime;
	}

	public void setBeginTime(Date beginTime) {
		this.beginTime = beginTime;
	}

	public Date getEndTime() {
		return endTime;
	}

	public void setEndTime(Date endTime) {
		this.endTime = endTime;
	}

	public Boolean getExamLimit() {
		return examLimit;
	}

	public void setExamLimit(Boolean examLimit) {
		this.examLimit = examLimit;
	}

	public Boolean getSpecialSettingsEnabled() {
		return specialSettingsEnabled;
	}

	public void setSpecialSettingsEnabled(Boolean specialSettingsEnabled) {
		this.specialSettingsEnabled = specialSettingsEnabled;
	}

	public ExamSpecialSettingsType getSpecialSettingsType() {
		return specialSettingsType;
	}

	public void setSpecialSettingsType(ExamSpecialSettingsType specialSettingsType) {
		this.specialSettingsType = specialSettingsType;
	}

	public String getExt1() {
		return ext1;
	}

	public void setExt1(String ext1) {
		this.ext1 = ext1;
	}

	public String getExt2() {
		return ext2;
	}

	public void setExt2(String ext2) {
		this.ext2 = ext2;
	}

	public String getExt3() {
		return ext3;
	}

	public void setExt3(String ext3) {
		this.ext3 = ext3;
	}

	public String getExt4() {
		return ext4;
	}

	public void setExt4(String ext4) {
		this.ext4 = ext4;
	}

	public String getExt5() {
		return ext5;
	}

	public void setExt5(String ext5) {
		this.ext5 = ext5;
	}

}
