package cn.com.qmth.examcloud.core.examwork.service;

import java.io.File;
import java.util.List;
import java.util.Map;
import java.util.Set;

import cn.com.qmth.examcloud.api.commons.enums.CURD;
import cn.com.qmth.examcloud.api.commons.exchange.PageInfo;
import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamEntity;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamIpLimitEntity;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamSpecialSettingsEntity;
import cn.com.qmth.examcloud.core.examwork.dao.enums.IpLimitType;
import cn.com.qmth.examcloud.core.examwork.service.bean.ExamInfo;
import cn.com.qmth.examcloud.core.examwork.service.bean.ExamIpLimitInfo;
import cn.com.qmth.examcloud.core.examwork.service.bean.ExamSpecialSettingsInfo;
import cn.com.qmth.examcloud.core.examwork.service.bean.orgip.OrgIpInfo;

/**
 * 类注释
 *
 * @author WANGWEI
 * @date 2018年8月17日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
public interface ExamService {

    /**
     * 保存考试
     *
     * @param examInfo
     * @return
     * @author WANGWEI
     */
    ExamEntity saveExam(ExamInfo examInfo, CURD es);

    /**
     * 保存考试机构设置
     *
     * @param examSpecialInfo
     * @return
     * @author WANGWEI
     */
    ExamSpecialSettingsEntity saveExamSpecialSettings(ExamSpecialSettingsInfo examSpecialInfo);

    /**
     * 查询考试机构属性
     *
     * @param examId
     * @param orgId
     * @param key
     * @return
     * @author WANGWEI
     */
    String getExamOrgProperty(Long examId, Long orgId, String key);

    /**
     * 查询考试学生属性
     *
     * @param examId
     * @param studentId
     * @param key
     * @return
     * @author WANGWEI
     */
    String getExamStudentProperty(Long examId, Long studentId, String key);

    /**
     * 导入学习中心设置
     *
     * @param examId
     * @param file
     * @return
     * @author WANGWEI
     */
    List<Map<String, Object>> importExamOrgSettings(Long examId, File file);

    /**
     * 查询考试属性
     *
     * @param examId
     * @param key
     * @return
     * @author WANGWEI
     */
    String getExamProperty(Long examId, String key);

    Map<Long, ExamEntity> getExamMapsByIds(Set<Long> examIds);

    /**
     * Description ip限制分页查询
     *
     * @param curPage
     * @param pageSize
     * @param examId
     * @param limitType
     * @param ip
     * @return PageInfo<ExamIpLimitInfo>
     */
    PageInfo<ExamIpLimitInfo> pageIpLimited(Integer curPage, Integer pageSize, Long examId, IpLimitType limitType, String ip);

    /**
     * Description ip限制列表查询
     *
     * @param examId
     * @param limitType
     * @param ip
     * @return List<ExamIpLimitEntity>
     */
    List<ExamIpLimitEntity> findAllIpLimits(Long examId, IpLimitType limitType, String ip);

    /***
     * Description 查询当前用户当前考试是否被限制
     *
     * @param realIp
     * @param examId
     * @param accessUser
     * @return boolean
     */
    boolean getIpLimited(String realIp, Long examId, User accessUser);

    /***
     * Description 保存ip限制信息
     *
     * @param ids
     * @param entity
     * @return void
     */
    void saveIpLimit(String ids, ExamIpLimitEntity entity);

    /***
     * Description 根据ids批量删除ip限制(格式如：1,2,3)
     *
     * @param ids
     * @return void
     */
    void deleteIpLimit(String ids);

    /***
     * Description 根据考试id批量删除
     *
     * @param examId
     * @return void
     */
    void deleteIpLimit(Long examId);

    /***
     * Description 获取整体控制跟学校中心控制
     *
     * @param examId
     * @return Map<String,Object>
     */
    Map<String, Boolean> getIpLimitedProperty(Long examId);

    /***
     * Description 更新整体控制和学习中心控制
     *
     * @param examId
     * @param map
     * @return void
     */
    void updateIpLimitedProperty(Long examId, Map<String, Object> map);

    /***
     * Description 批量保存ip限制信息
     *
     * @param list
     * @return void
     */
    void saveAllIpLimits(List<ExamIpLimitEntity> list);

	void addIpLimitByOrgIp(Long examId,List<OrgIpInfo> ips);
}
