/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-08-31 09:28:53.
 * *************************************************
 */

package cn.com.qmth.examcloud.core.examwork.service;

import cn.com.qmth.examcloud.api.commons.enums.NoticeReceiverRuleType;
import cn.com.qmth.examcloud.api.commons.enums.NoticeStatus;
import cn.com.qmth.examcloud.api.commons.exchange.PageInfo;
import cn.com.qmth.examcloud.api.commons.security.bean.UserType;
import cn.com.qmth.examcloud.core.examwork.dao.entity.NoticeRulePublishProgressEntity;
import cn.com.qmth.examcloud.core.examwork.service.bean.*;

import java.util.List;

/**
 * 用户通知接口
 */
public interface NoticeService {

    /**
     * 获取用户通知列表
     *
     * @param query
     * @return
     */
    List<UserNoticeInfo> getNoticeList(UserNoticeInfoQuery query);

    /**
     * 更新用户的通知状态为已读
     *
     * @param noticeId 通知id（多个以逗号分隔）
     * @param userType 用户类型
     * @param userId   用户id
     * @return
     */
    int updateNoticeReadStatus(String noticeId, UserType userType, Long userId);

    /**
     * 分页获取通知列表
     *
     * @param curPage
     * @param pageSize
     * @param infoQuery
     * @return
     */
    PageInfo<NoticeInfo> getPagedNoticeList(Integer curPage, Integer pageSize,
                                            NoticeInfoQuery infoQuery);

    /**
     * 删除通知
     *
     * @param rootOrgId
     * @param noticeIdList
     * @return
     */
    int deleteNotice(Long rootOrgId, List<Long> noticeIdList);

    /**
     * 添加通知
     *
     * @param addNoticeInfo
     * @return
     */
    int addNotice(AddNoticeInfo addNoticeInfo);

    /**
     * 更新通知信息
     *
     * @param info
     * @return
     */
    NoticeRulePublishProgressEntity updateNotice(UpdateNoticeInfo info);

    /**
     * 获取待处理的通知规则发布进度集合
     *
     * @return
     */
    List<NoticeRulePublishProgressEntity> getToBeDisposedNoticeRulePublishProgressList();

    /**
     * 处理待发送的用户通知(自动服务调用)
     *
     * @param startUserId
     * @return 下次请求的用户id
     */
    Long disposePublishingUserNotice(Long startUserId, NoticeRulePublishProgressEntity rulePublishProcess);

    /**
     * 处理过期的通知数据(自动服务调用)
     */
    void disposeOverdueNotice();

    /**
     * 更新通知状态
     *
     * @param noticeId
     * @param noticeStatus
     */
    void updateNoticeStatus(Long noticeId, NoticeStatus noticeStatus);

    void sendMsg(Long id);

    void recallMsg(Long id);
}