package cn.com.qmth.examcloud.core.examwork.service.cache;

import cn.com.qmth.examcloud.commons.helpers.DynamicEnum;
import cn.com.qmth.examcloud.commons.helpers.DynamicEnumManager;
import cn.com.qmth.examcloud.core.examwork.base.enums.ExamProperty;
import cn.com.qmth.examcloud.core.examwork.dao.ExamOrgPropertyRepo;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamOrgPropertyEntity;
import cn.com.qmth.examcloud.core.examwork.service.ExamService;
import cn.com.qmth.examcloud.support.CacheConstants;
import cn.com.qmth.examcloud.support.cache.bean.ExamOrgPropertyCacheBean;
import cn.com.qmth.examcloud.web.cache.RandomObjectRedisCache;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class ExamOrgPropertyCache extends RandomObjectRedisCache<ExamOrgPropertyCacheBean> {

    @Autowired
    ExamService examService;

    @Autowired
    ExamOrgPropertyRepo examOrgPropertyRepo;

    @Override
    public ExamOrgPropertyCacheBean loadFromResource(Object... keys) {
        Long examId = (Long) keys[0];
        Long orgId = (Long) keys[1];
        String key = (String) keys[2];

        DynamicEnumManager manager = ExamProperty.getDynamicEnumManager();
        DynamicEnum de = manager.getByName(key);

        ExamOrgPropertyCacheBean b = new ExamOrgPropertyCacheBean();
        b.setExamId(examId);
        b.setOrgId(orgId);
        b.setKey(key);

        ExamOrgPropertyEntity examOrgPropertyEntity = examOrgPropertyRepo
                .findByExamIdAndOrgIdAndKeyId(examId, orgId, de.getId());
        if (null != examOrgPropertyEntity) {
            b.setValue(examOrgPropertyEntity.getValue());
        } else {
            b.setHasValue(false);
        }

        return b;
    }

    @Override
    protected String getKeyPrefix() {
        return CacheConstants.CACHE_E_EXAM_ORG_PROP;
    }

    @Override
    protected int getTimeout() {
        // 10s
        return 10;
    }

}
