package cn.com.qmth.examcloud.core.examwork.service.cache;

import cn.com.qmth.examcloud.api.commons.enums.SubmitType;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.util.DateUtil;
import cn.com.qmth.examcloud.core.examwork.dao.ExamStageRepo;
import cn.com.qmth.examcloud.core.examwork.dao.ExamStageSettingRepo;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamStageEntity;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamStageSettingEntity;
import cn.com.qmth.examcloud.support.CacheConstants;
import cn.com.qmth.examcloud.support.cache.bean.ExamStageCacheBean;
import cn.com.qmth.examcloud.web.cache.RandomObjectRedisCache;
import cn.com.qmth.examcloud.web.helpers.GlobalHelper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class ExamStageCache extends RandomObjectRedisCache<ExamStageCacheBean> {

    @Autowired
    ExamStageRepo examStageRepo;

    @Autowired
    ExamStageSettingRepo examStageSettingRepo;

    @Override
    public ExamStageCacheBean loadFromResource(Object... keys) {
        Long examId = (Long) keys[0];
        Long examStageId = (Long) keys[1];

        if (null == examId) {
            throw new StatusException("002001", "考试id不允许为空");
        }
        if (null == examStageId) {
            throw new StatusException("002002", "场次id不允许为空");
        }

        ExamStageCacheBean result = new ExamStageCacheBean();
        ExamStageEntity examStage = GlobalHelper.getEntity(examStageRepo, examStageId, ExamStageEntity.class);
        if (null == examStage) {
            result.setHasValue(false);
            return result;
        }

        result.setId(examStage.getId());
        result.setStageOrder(examStage.getStageOrder());
        result.setRootOrgId(examStage.getRootOrgId());
        result.setExamId(examStage.getExamId());
        result.setEnable(examStage.getEnable());
        result.setStartTime(DateUtil.parse(DateUtil.format(examStage.getStartTime(),
                DateUtil.DatePatterns.CHINA_DEFAULT), DateUtil.DatePatterns.CHINA_DEFAULT));
        result.setEndTime(DateUtil.parse(DateUtil.format(examStage.getEndTime(),
                DateUtil.DatePatterns.CHINA_DEFAULT), DateUtil.DatePatterns.CHINA_DEFAULT));
        result.setStartExamStatus(examStage.getStartExamStatus().name());

        //通用设置or特殊设置
        Boolean specialSetting = null == examStage.getSpecialSetting() ? false : examStage.getSpecialSetting();

        result.setSpecialSetting(specialSetting);

        //如果是特殊设置，取取场次表中设置
        if (specialSetting) {
            result.setSubmitType(examStage.getSubmitType().name());
            result.setSubmitDuration(examStage.getSubmitDuration());
        }
        //如果是通用设置，则取通用设置表中的设置
        else {
            ExamStageSettingEntity examStageSetting = examStageSettingRepo.findByExamId(examId);

            //如果未找到通用设置，则初始化通用设置
            if (null == examStageSetting) {
                examStageSetting = new ExamStageSettingEntity();
                examStageSetting.setExamId(examId);
                examStageSetting.setSubmitType(SubmitType.NORMAL);//默认正常交卷
                examStageSettingRepo.save(examStageSetting);
            }

            result.setSubmitType(examStageSetting.getSubmitType().name());
            result.setSubmitDuration(examStageSetting.getSubmitDuration());
        }

        return result;
    }

    @Override
    protected String getKeyPrefix() {
        return CacheConstants.CACHE_E_EXAM_STAGE;
    }

    @Override
    protected int getTimeout() {
        // 10s
        return 10;
    }

}
