package cn.com.qmth.examcloud.core.examwork.service.cache;

import cn.com.qmth.examcloud.api.commons.enums.SubmitType;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.util.DateUtil;
import cn.com.qmth.examcloud.core.examwork.dao.ExamStageRepo;
import cn.com.qmth.examcloud.core.examwork.dao.ExamStageSettingRepo;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamStageEntity;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamStageSettingEntity;
import cn.com.qmth.examcloud.support.CacheConstants;
import cn.com.qmth.examcloud.support.cache.bean.ExamStageCacheBean;
import cn.com.qmth.examcloud.support.cache.bean.ExamStagesCacheBean;
import cn.com.qmth.examcloud.web.cache.RandomObjectRedisCache;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service
public class ExamStagesCache extends RandomObjectRedisCache<ExamStagesCacheBean> {

    @Autowired
    ExamStageRepo examStageRepo;

    @Autowired
    ExamStageSettingRepo examStageSettingRepo;

    @Override
    public ExamStagesCacheBean loadFromResource(Object... keys) {
        Long examId = (Long) keys[0];

        if (null == examId) {
            throw new StatusException("002001", "考试id不允许为空");
        }

        ExamStagesCacheBean result = new ExamStagesCacheBean();
        List<ExamStageEntity> entityList = examStageRepo.findByExamId(examId);
        if (null == entityList || entityList.isEmpty()) {
            result.setHasValue(false);
            return result;
        }

        List<ExamStageCacheBean> beanList = new ArrayList<>();
        for (ExamStageEntity esEntity : entityList) {
            ExamStageCacheBean esBean = new ExamStageCacheBean();
            esBean.setId(esEntity.getId());
            esBean.setRootOrgId(esEntity.getRootOrgId());
            esBean.setExamId(esEntity.getExamId());
            esBean.setStageOrder(esEntity.getStageOrder());
            esBean.setStartTime(DateUtil.parse(DateUtil.format(esEntity.getStartTime(),
                    DateUtil.DatePatterns.CHINA_DEFAULT), DateUtil.DatePatterns.CHINA_DEFAULT));
            esBean.setEndTime(DateUtil.parse(DateUtil.format(esEntity.getEndTime(),
                    DateUtil.DatePatterns.CHINA_DEFAULT), DateUtil.DatePatterns.CHINA_DEFAULT));
            esBean.setEnable(esEntity.getEnable());
            esBean.setStartExamStatus(esEntity.getStartExamStatus().name());

            //通用设置or特殊设置
            Boolean specialSetting = null == esEntity.getSpecialSetting() ? false : esEntity.getSpecialSetting();

            esBean.setSpecialSetting(specialSetting);

            //如果是特殊设置，取取场次表中设置
            if (specialSetting) {
                esBean.setSubmitType(esEntity.getSubmitType().name());
                esBean.setSubmitDuration(esEntity.getSubmitDuration());
            }
            //如果是通用设置，则取通用设置表中的设置
            else {
                ExamStageSettingEntity examStageSetting = examStageSettingRepo.findByExamId(examId);

                //如果未找到通用设置，则初始化通用设置
                if (null == examStageSetting) {
                    examStageSetting = new ExamStageSettingEntity();
                    examStageSetting.setExamId(examId);
                    examStageSetting.setSubmitType(SubmitType.NORMAL);//默认正常交卷
                    examStageSettingRepo.save(examStageSetting);
                }

                esBean.setSubmitType(examStageSetting.getSubmitType().name());
                esBean.setSubmitDuration(examStageSetting.getSubmitDuration());
            }
            beanList.add(esBean);
        }

        result.setExamStages(beanList);

        return result;
    }

    @Override
    protected String getKeyPrefix() {
        return CacheConstants.CACHE_E_EXAM_STAGES;
    }

    @Override
    protected int getTimeout() {
        // 10s
        return 10;
    }

}
